/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import org.netbeans.api.project.Project;
import org.openide.util.Lookup;

/**
 *
 * @author johan
 */
public abstract class AbstractEntityManager<E extends Entity> implements EntityManager<E> {
    private static final EntityCache CACHE = Lookup.getDefault().lookup(EntityCache.class);
    
    @Override
    public boolean supports(ItemIdType itemIdType) {
        if (getSupportedTypes() == null) {
            return false;
        }
        return getSupportedTypes().contains(itemIdType);
    }

    /**
     * Find an entity instance by its URI.
     * 
     * @param <E>
     * @param uri
     * @return the entity instance or null when not found
     */
    @Override
    // FIXME: doesn't work for nested entities anymore now that the generic type has been moved from method to class level.
    public E find(AgroURI uri) {
        
        E entity = (E) CACHE.get(uri);
        
        if (entity == null) {
            // load() implementations are responsible for adding the entity to the cache
            entity = load(uri);
        }
                
        return entity;
    }
    
    /**
     * add an entity instance to the cache.
     * override in subclasses to handle cascading to nested/embedded entities.
     * 
     * @param entity 
     */
    protected void addToCache(Entity entity) {
        CACHE.add(entity);
    }
    
    /**
     * remove an entity instance from the cache.
     * override in subclasses to handle cascading to nested/embedded entities.
     * 
     * @param entity 
     */
    protected void removeFromCache(Entity entity) {
        CACHE.remove(entity);
    }
    
    /**
     * Finds or creates a cached instance for the input entity's URI.
     * 
     * helps ensure that a single entity instance is used for each URI key.
     * 
     * @param entity
     * @return either the already cached instance for the input entity's URI, 
     *          or, if none was found, the input entity itself.
     *          Should never return null for non-null input.
     */
    protected E getCachedInstanceFor(E entity) {
        E cachedInstance = (E) CACHE.get(entity.getURI());
        if (cachedInstance == null) {
            addToCache(entity);
            cachedInstance = entity;
        }
        return cachedInstance;
    }
       
    protected E load(AgroURI uri) {
        Project project = EnterpriseIdProvider.ProjectFinder.find(uri);
        return load(uri, project);
    }
    
    protected abstract E load(AgroURI uri, Project project);
    
//    public void save(Entity entity) {
//        Project project = EnterpriseIdProvider.ProjectFinder.find(entity.getURI());
//        save(entity, project);
//    }
//
//    protected abstract void save(Entity entity, Project project);

}
