/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import java.util.Collection;
import java.util.logging.Logger;
import org.openide.util.Lookup;

/**
 * Generator for generating unique entity id's
 *
 * There should only be one implementation available in the application. Use the
 * Locator class to get an instance to guarantee only one instance is loaded in
 * the application
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public interface EntityIdGenerator {
    
    // default key for user defined data in e.g. crop storage
    String USER_DEFINED_KEY = "UserDefined";

    /**
     * Generates a unique entity id for this installation. Uniqueness across
     * application installations or when user directory is cleared is not part
     * of the implementation contract.
     *
     * @param itemIdType entity type
     * @return unique entity id
     */
    long generateId(ItemIdType itemIdType);
    long generateId(String key);

    public static class Locator {

        private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.model.FieldIdGenerator");
        private static final EntityIdGenerator dummyIdGenerator = new DummyIdGenerator();

        /**
         * Failsafe convenience method to get a EntityIdGenerator implementation.
         *
         * @return
         */
        public static EntityIdGenerator getInstance() {
            Collection<? extends EntityIdGenerator> generators = Lookup.getDefault().lookupAll(EntityIdGenerator.class);
            if (generators.size() > 1) {
                throw new RuntimeException("FIXME: More then one unique id generators present in the application!");
            }
            if (generators.isEmpty()) {
                LOGGER.warning("FIXME: No EntityIdGenerator implementation found, returning dummy implementation");
                return dummyIdGenerator;
            }
            return generators.iterator().next();
        }
    }

    static final class DummyIdGenerator implements EntityIdGenerator {

        static long id = 0l;

        @Override
        public long generateId(ItemIdType itemIdType) {
            return generateId(itemIdType.name());
        }

        @Override
        public long generateId(String key) {
            return id++;
        }
        
        
    }
}
